import { AppPage } from "./app.po";
import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { SideNav } from "./sidenav.po";
import { browser } from "protractor";

/**
 * Load a session containing 4 calculators, having multiple linked parameters
 * from one to another
 * @TODO les valeurs des Select sont comparées au français, pas très générique :/
 */
describe("ngHyd − load session with multiple linked parameters − ", () => {
  let startPage: AppPage;
  let listPage: ListPage;
  let calcPage: CalculatorPage;
  let navbar: Navbar;
  let sidenav: SideNav;

  function init() {
      startPage = new AppPage();
      calcPage = new CalculatorPage();
      navbar = new Navbar();
      sidenav = new SideNav();
      listPage = new ListPage();
  }
  beforeEach(init);

   it("when loading session-liens-spaghetti.json, all links should point to the right target", async () => {
    await startPage.navigateTo();

    await navbar.clickMenuButton();
    await browser.sleep(200);

    await sidenav.clickLoadSessionButton();
    await browser.sleep(200);

    await sidenav.loadSessionFile("./session-liens-spaghetti.json");
    await browser.sleep(500);

    expect(await navbar.getAllCalculatorTabs().count()).toBe(5);

    // 1. check Section paramétrée
    await navbar.clickCalculatorTab(0);

    // check target params values
    const sp_lb = calcPage.getSelectById("linked_LargeurBerge");
    const sp_lbv = await calcPage.getSelectValueText(sp_lb);
    expect(sp_lbv).toEqual("L (Ouvrages, ouvrage 3)");

    // 2. check Passe à macro-rugosités
    await navbar.clickCalculatorTab(1);

    // check target params values
    const mr_b = calcPage.getSelectById("linked_B");
    const mr_bv = await calcPage.getSelectValueText(mr_b);
    expect(mr_bv).toEqual("L (résultat de Ouvrages)");

    const mr_q = calcPage.getSelectById("linked_Q");
    const mr_qv = await calcPage.getSelectValueText(mr_q);
    expect(mr_qv).toEqual("Q (R. uniforme)");

    // 3. check Lois d'ouvrages
    await navbar.clickCalculatorTab(2);

    // check target params values
    const lo_z1 = calcPage.getSelectById("linked_Z1");
    const lo_z1v = await calcPage.getSelectValueText(lo_z1);
    expect(lo_z1v).toEqual("ZF1 (Macro-rugo.)");

    const lo_l = calcPage.getSelectById("linked_L"); // attention ID non unique, voir nghyd#173
    const lo_lv = await calcPage.getSelectValueText(lo_l);
    expect(lo_lv).toEqual("L (résultat de Ouvrages)");

    const lo_w = calcPage.getSelectById("linked_W"); // attention ID non unique, voir nghyd#173
    const lo_wv = await calcPage.getSelectValueText(lo_w);
    expect(lo_wv).toEqual("W (Ouvrages, ouvrage 2)");

    const lo_cd = calcPage.getSelectById("linked_Cd"); // attention ID non unique, voir nghyd#173
    const lo_cdv = await calcPage.getSelectValueText(lo_cd);
    expect(lo_cdv).toEqual("Cd (Ouvrages, ouvrage 1)");

    // 4. check Régime uniforme
    await navbar.clickCalculatorTab(3);

    // check target params values
    const lo_q = calcPage.getSelectById("linked_Q");
    const lo_qv = await calcPage.getSelectValueText(lo_q);
    expect(lo_qv).toEqual("CvQT (Déver. dénoyés, résultat complémentaire)");

    // 5. check Déver. dénoyés
    await navbar.clickCalculatorTab(4);

    // check target params values
    const lo_br = calcPage.getSelectById("linked_BR");
    const lo_brv = await calcPage.getSelectValueText(lo_br);
    expect(lo_brv).toEqual("LargeurBerge (Sec. param.)");
  });

});
